@@ -120,11 +120,15 @@ swapoff_usage(FILE *fp, int n) {
 static int numSwaps;
 static char **swapFiles;	/* array of swap file and partition names */
 
+#define DELETED_SUFFIX		"\\040(deleted)"
+#define DELETED_SUFFIX_SZ	(sizeof(DELETED_SUFFIX) - 1)
+
 static void
 read_proc_swaps(void) {
 	FILE *swaps;
 	char line[1024];
 	char *p, **q;
+	size_t sz;
 
 	numSwaps = 0;
 	swapFiles = NULL;
@@ -152,7 +156,17 @@ read_proc_swaps(void) {
 		 * This will fail with names with embedded spaces.
 		 */
 		for (p = line; *p && *p != ' '; p++);
-		*p = 0;
+		*p = '\0';
+
+		/* the kernel can use " (deleted)" suffix for paths
+		 * in /proc/swaps, we have to remove this junk.
+		 */
+		sz = strlen(line);
+		if (sz > DELETED_SUFFIX_SZ) {
+		       p = line + (sz - DELETED_SUFFIX_SZ);
+		       if (strcmp(p, DELETED_SUFFIX) == 0)
+			       *p = '\0';
+		}
 
 		q = realloc(swapFiles, (numSwaps+1) * sizeof(*swapFiles));
 		if (q == NULL)
@@ -347,14 +361,14 @@ swap_get_size(const char *hdr, const char *devname, unsigned int pagesize)
 		last_page = swab32(s->last_page);
 	}
 	if (verbose)
-		warnx(_("%s: found %sswap v%d signature string"
-				" for %d KiB PAGE_SIZE\n"),
+		warnx(_("%s: found swap signature: version %d, "
+			"page-size %d, %s byte order"),
 			devname,
-			flip ? "other-endian " : "",
 			swap_version,
-			pagesize / 1024);
+			pagesize / 1024,
+			flip ? _("different") : _("same"));
 
-	return (last_page + 1) * pagesize;
+	return ((unsigned long long) last_page + 1) * pagesize;
 }
 
 static int
@@ -791,6 +805,9 @@ main(int argc, char *argv[]) {
 
 	if (streq(progname, "swapon"))
 		return main_swapon(argc, argv);
-	else
+	else if (streq(progname, "swapoff"))
 		return main_swapoff(argc, argv);
+
+	errx(EXIT_FAILURE, _("'%s' is unsupported program name "
+			"(must be 'swapon' or 'swapoff')."), progname);
 }
